﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO.Ports;
using System.IO;
using System.Globalization;

namespace RemSwr
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
            culture = new CultureInfo("en-US");
            cbComPort.SelectedIndex = 0;
            try
            {
                TextReader tr = new StreamReader(inifile);
                string portname = tr.ReadLine();
                tr.Close();
                
                foreach (object x in cbComPort.Items)
                {
                    if (x.ToString() == portname)
                    {
                        cbComPort.SelectedItem = x;
                        break;
                    }
                }
            }
            catch { }

            timer1 = new Timer();
            timer1.Interval = 100;    // 100 msec
            timer1.Tick += new EventHandler(timer1_Tick);
            timer1.Start();
        }

        const int timeout = 100;   // timeout 10 sec, (100msec base)

        Timer timer1;
        SerialPort port;
        string rxbuf = "";
        volatile int toPwr = 0;  // timeouts
        volatile int toSwr = 0;
        double[] pwrBuf = new double[10];
        bool warning = false;
        string inifile = @"RemSwr.ini";
        string power = "";
        string swr = "";
        int powerbar = 0;
        int swrbar = 0;

        CultureInfo culture;

        void port_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            int chr;
            try
            {
                while ((chr = port.ReadByte()) >= 0)
                {
                    if ((chr == 0x0a) || (chr == 0x0d))
                    {
                        ProcessData(rxbuf);
                        rxbuf = "";
                    }
                    else
                        rxbuf += (char)chr;
                }
            }
            catch { }
        }

        void ProcessData(string data)
        {
            data = data.Trim();
            if (data.Length < 2)
                return;
            if (data.StartsWith("Pw"))
            {
                power = data.Substring(2);
                toPwr = 0;
                string value = "";
                int i = 0;
                while ((i < 10) && ((power[i] >= '0') && (power[i] <= '9') || (power[i] == '.')))
                {
                    value += power[i++];
                }

                try
                {
                    double pwr = Double.Parse(value,culture.NumberFormat);
                    double maxPower = 0.0;
                    for (i = 0; i < 9; i++)
                    {
                        pwrBuf[i] = pwrBuf[i + 1];  // shift buffer
                        if (pwrBuf[i] > maxPower)
                            maxPower = pwrBuf[i];
                    }
                    pwrBuf[9] = pwr;  // enter Fifo buffer
                    if (pwr > maxPower)
                        maxPower = pwr;

                    powerbar =(maxPower > 0)? (int)(pwr/maxPower*100): 0;  // bar indicator
                }
                catch { }
            }
            else if (data.StartsWith("SWR"))
            {
                swr = data.Substring(3);
                toSwr = 0;
                try
                {
                    double swrd = Double.Parse(swr, culture.NumberFormat);
                    swrbar = swrd > 1 ? (int)(100 - 200 / (swrd + 1)) : 0;   // SWR bar
                }
                catch { }
            }
        }

        void timer1_Tick(object sender, EventArgs e)
        {
            if ((port == null) || !port.IsOpen)
            {
                string sel = cbComPort.SelectedItem.ToString();
                port = new SerialPort(sel, 9600);
                port.DataReceived += new SerialDataReceivedEventHandler(port_DataReceived);
                rxbuf = "";
                for (int i = 0; i < 10; i++)
                    pwrBuf[i] = 0.0;
                try
                {
                    port.Open();
                }
                catch (Exception ex)
                {
                    if (!warning)
                    {
                        warning = true;
                        MessageBox.Show(ex.Message, "Error"); 
                    }
                }
            }

            if (toPwr++ >= timeout)
            {
                toPwr = timeout;
                lbPwr.Text = "???";
                barPwr.Value = 0;
            }
            else
            {
                lbPwr.Text = power;
                barPwr.Value = powerbar;
            }

            if (toSwr++ >= timeout)
            {
                toSwr = timeout;
                lbSwr.Text = "???";
                barSwr.Value = 0;
            }
            else
            {
                lbSwr.Text = swr;
                barSwr.Value = swrbar;
            }
        }

        private void comboBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            warning = false;    // warning box will be shown upon next event
            if (port != null)   // if connected, disconnect old port
            {
                if (port.IsOpen)
                    port.Close();
                port = null;
            }
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (port != null)   // if connected, disconnect
            {
                if (port.IsOpen)
                    port.Close();
                port = null;
            }

            try
            {
                TextWriter wr = new StreamWriter(inifile);
                wr.WriteLine(cbComPort.SelectedItem.ToString());
                wr.Close();
            }
            catch { }
        }


    }
}
