﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace SwrCnfg
{
    public partial class Form1 : Form
    {
        bool connected = false;
        bool lastConnected = false;
        int fwd, rev, probe;
        int timeout;
        internal ComDlg comdlg = null;
        DataTable tbl;
        byte SwrRangebyte = 0;
        int SwrScale = 1000;
        double maxPwr;
        bool update = false;

        public Form1()
        {
            InitializeComponent();
            cbCOM.SelectedIndex = 0;
            cbUnit.SelectedIndex = 0;
            serialPort1.NewLine = "\r\n";
            tbl = dataSet1.Tables[0];
            fwd = -1;
            rev = -1;
            probe = -1;
            for (int i = 0; i < 17; i++)
            {
                DataRow row = tbl.NewRow();
                row[0] = i * 64;
                tbl.Rows.Add(row);
            }
        }

        private void serialPort1_DataReceived(object sender, System.IO.Ports.SerialDataReceivedEventArgs e)
        {
            try
            {
                string line = serialPort1.ReadLine();
                string sprobe = numProbe.Value.ToString();
                if (!connected && (line.IndexOf("SWR10") >= 0))
                {
                    connected = true;

                }
                else if (line.StartsWith("F"))
                {
                    fwd = int.Parse(line.Substring(1));
                }
                else if (line.StartsWith("R"))
                {
                    rev = int.Parse(line.Substring(1));
                }
                else if (line.StartsWith("P"))
                {
                    probe = int.Parse(line.Substring(1));
                }
                else if (line.StartsWith(sprobe))
                {
                    update = true;
                    if (line[1] == 'G') // range
                    {
                        SwrRangebyte = byte.Parse(line.Substring(2));
                        int dp = 4 - (SwrRangebyte & 0x0F);
                        maxPwr = (double)SwrScale;
                        for (int i = 0; i < dp; i++)
                        {
                            maxPwr /= 10;
                        }
                    }
                    if (line[1] == 'S') // scale
                    {
                        SwrScale = int.Parse(line.Substring(2));
                        int dp = 4 - (SwrRangebyte & 0x0F);
                        maxPwr = (double)SwrScale;
                        for (int i = 0; i < dp; i++)
                        {
                            maxPwr /= 10;
                        }
                        
                    }
                    if (line[1] == 'C') // correction
                    {
                        int idx = line[2] - 'A' + 1;
                        if ((idx > 0) && (idx <= 16))
                        {
                            DataRow dr = tbl.Rows[idx];
                            short corr = (short)byte.Parse(line.Substring(3));
                            if (corr > 127)
                                corr -= 256;
                            dr[4] = corr;
                            dr[1] = (short)dr[0] + corr;  // corrected ADC value
                            double vr;
                            dr[2] = vr = ((double)(short)dr[1]) / 1024; // normalized voltage
                            dr[3] = maxPwr * vr * vr;   // power
                        }
                    }
                }
                else if (line.StartsWith("W"))  // write acknowledge
                {

                }
                timeout = 0;
                if (comdlg != null)
                    comdlg.AppendText(line);
            }
            catch { }
        }

        private void butCOM_Click(object sender, EventArgs e)
        {
            comdlg = new ComDlg(this);
            comdlg.Show(this);
        }

        private void cbCOM_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                serialPort1.Close();
                serialPort1.PortName = cbCOM.Text;
                serialPort1.Open();
            }
            catch
            {
                MessageBox.Show("Failure to access " + serialPort1.PortName + " port", "Error");
            }
        }


        // refresh screen (update not possible from serial port thread)
        private void timer1_Tick(object sender, EventArgs e)
        {
            lbStatus.Text = connected ? "Status: Connected" : "Status: Disconnected";
            if (connected && !lastConnected)
            {
                serialPort1.WriteLine("set");
            }
            lastConnected = connected;

            lbProbe.Text = probe >=0 ? probe.ToString() : "???";
            int probenr = probe / 128;
            lbProbeNr.Text = probe >= 0 ? probenr.ToString() : "?";
            lbRev.Text = rev >= 0 ? rev.ToString() : "???";
            lbFwd.Text = fwd >= 0 ? fwd.ToString() : "???";

            timeout++;
            if (timeout > 30)
            {
                connected = false;
                lastConnected = false;
                fwd = -1;
                rev = -1;
                probe = -1;
            }

            if (update)
            {
                update = false;
                cbUnit.SelectedIndex = SwrRangebyte >> 4;
                txtMaxPower.Text = maxPwr.ToString();
                dxChart11.Invalidate();
            }
        }

        private void rbVoltage_CheckedChanged(object sender, EventArgs e)
        {
            if (rbVoltage.Checked)
            {
                dataGridView1.Columns[3].ReadOnly = false;
                dataGridView1.Columns[4].ReadOnly = true;
                txtMaxPower.Enabled = true;
            }
            else
            {
                dataGridView1.Columns[3].ReadOnly = true;
                dataGridView1.Columns[4].ReadOnly = false;
                txtMaxPower.Enabled = false;
            }
        }

        // check correction in range
        private bool CheckCorr()
        {
            try
            {
                bool ok = true;
                foreach (DataRow dr in tbl.Rows)
                {
                    if (((short)dr[4] < -128) || ((short)dr[4] > 127))
                    {
                        ok = false;
                        break;
                    }
                }
                if (!ok)
                    MessageBox.Show("Correction value out of range (-128 to 127)", "Warning");
                return ok;
            }
            catch(Exception ex)
            {
                MessageBox.Show(ex.Message, "Error");
                return false;
            }
        }

        private void butRecalc_Click(object sender, EventArgs e)
        {
            tbl.Rows[0][2] = 0.0;
            tbl.Rows[0][3] = 0.0;
            double lastval = 0.0;
            if (rbVoltage.Checked)
            {
                bool first = true;
                foreach (DataRow dr in tbl.Rows)
                {
                    if (!first) // ignore first row
                    {
                        if (dr.IsNull(2))
                        {
                            MessageBox.Show("Table entry not complete", "Error");
                            return;
                        }
                        double volt = (double)dr[2];
                        if (volt <= lastval)
                        {
                            MessageBox.Show("Voltage not in growing order", "Error");
                            return;
                        }
                        lastval = volt;
                    }
                    first = false;
                }

                double max = (double)tbl.Rows[tbl.Rows.Count - 1][2];
                double range = 0;
                try
                {
                    range = double.Parse(txtMaxPower.Text);
                    if (range <= 0)
                        throw (new Exception());
                }
                catch
                {
                    MessageBox.Show("Enter valid power value", "Error");
                    return;
                }
                foreach (DataRow dr in tbl.Rows)
                {
                    dr[1] = (int)(((double)dr[2]/max) * 1024);  // corrected ADC value
                    dr[4] = (short)dr[1] - (short)dr[0];
                    dr[3] = range * ((double)dr[2] / max) * ((double)dr[2] / max);  // calculated power
                }
            }
            else
            {
                bool first = true;
                foreach (DataRow dr in tbl.Rows)
                {
                    if (!first) // ignore first row
                    {
                        if (dr.IsNull(3))
                        {
                            MessageBox.Show("Table entry not complete", "Error");
                            return;
                        }
                        double watt = (double)dr[3];
                        if (watt <= lastval)
                        {
                            MessageBox.Show("Power not in growing order", "Error");
                            return;
                        }
                        lastval = watt;
                    }
                    first = false;
                }

                double max = (double)tbl.Rows[tbl.Rows.Count - 1][3];
                foreach (DataRow dr in tbl.Rows)
                {
                    dr[2] = Math.Sqrt((double)dr[3] / max);  // calculated voltage
                    dr[1] = (int)((double)dr[2] * 1024);  // corrected ADC value
                    dr[4] = (short)dr[1] - (short)dr[0];
                }
            }
            dxChart11.Invalidate();
            butSend.Enabled = CheckCorr();
        }

        private void dataGridView1_DataError(object sender, DataGridViewDataErrorEventArgs e)
        {
            MessageBox.Show(e.Exception.Message, "Error");

        }

        private void butRead_Click(object sender, EventArgs e)
        {
            if (connected)
            {
                tbl.Rows[0][0] = 0;
                tbl.Rows[0][1] = 0;
                tbl.Rows[0][2] = 0.0f;
                tbl.Rows[0][3] = 0.0f;
                tbl.Rows[0][4] = 0;
                serialPort1.WriteLine("r" + numProbe.Value);
            }
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (connected)
                serialPort1.WriteLine("q");
        }

        private void butSend_Click(object sender, EventArgs e)
        {
            if (CheckCorr() || true)    //simulation
            {
                bool ok;
                double pwr = double.Parse(txtMaxPower.Text);
                if (pwr <= 0)
                {
                    MessageBox.Show("Wrong power value", "Error");
                    return;
                }
                do
                {
                    ok = true;
                    if ((pwr >= 1000) && (cbUnit.SelectedIndex < cbUnit.Items.Count - 1))
                    {
                        pwr /= 1000.0f;
                        cbUnit.SelectedIndex++;
                        ok = false;
                    }
                    if ((pwr < 1.0f) && (cbUnit.SelectedIndex > 0))
                    {
                        pwr *= 1000.0f;
                        cbUnit.SelectedIndex--;
                        ok = false;
                    }
                    
                }
                while (!ok);
                txtMaxPower.Text = pwr.ToString();

                byte decP = 4;
                while (pwr < 1000.0f)
                {
                    pwr *= 10;  // pwr must be between 1000 and 9999
                    decP--;
                }
                SwrRangebyte = (byte)(cbUnit.SelectedIndex * 16);   // range byte: bits 0-3 = decimal place, bits 4-7 = unit
                SwrRangebyte |= decP;

                Cursor cur = this.Cursor;
                Cursor = Cursors.WaitCursor;

                // "wr" - EEPROM write; example wr1222 : write range for probe 1 ('1') value 222 ('222')
                string cmd = "wr" + numProbe.Value + SwrRangebyte;
                serialPort1.WriteLine(cmd);
                System.Threading.Thread.Sleep(500); // better wait for confirmation, but... Keep it simple, we have enough time.

                SwrScale = (int)pwr;    // normalized value between 1000 and 9999

                // "ws" - EEPROM write; example ws11235 : write scale for probe 1 ('1') value 1235 ('1235')
                cmd = "ws" + numProbe.Value + SwrScale;
                serialPort1.WriteLine(cmd);
                System.Threading.Thread.Sleep(500);

                for(int i = 0; i<16; i++)
                {
                    DataRow dr = tbl.Rows[i+1];
                    // "wp" - EEPROM write; example wp1c123 : write point 2 ('c') of probe 1 ('1') value 123 ('123')
                    short par = (short)dr[4];
                    if (par < 0)
                        par += 256;
                    cmd = "wp" + numProbe.Value + (char)(i + 'a') + par;
                    serialPort1.WriteLine(cmd);
                    System.Threading.Thread.Sleep(500);
                }
                Cursor = cur;
                MessageBox.Show("Configuration transferred", "Confirmation");
            }
        }

        private void butHelp_Click(object sender, EventArgs e)
        {
            SwrHelp dlg = new SwrHelp();
            dlg.Show();
        }

    }
}
